//////////////////////////////////////////////////////////////////////
// VcCom
// www.vccom.net
// VcCom API VC++ 6.0 Example Code
// Version 1.00
// Date 01 Aug 2004
/////////////////////////////////////////////////////////////////////
// Description
//
// This header file forms part of the VcCom API VC++ 6.0 Example Code
// in order to provide example usage of the VcCom API which is
// available from www.objectconsultans.co.uk
/////////////////////////////////////////////////////////////////////
// How To Use
//
// Please refer to the VcCom API VC++ 6.0 Example Code documentation
/////////////////////////////////////////////////////////////////////

// VcComClientWrapper.h

#pragma once

#include <string>
#import "VcCom.exe"

/////////////////////////////////////////////////////////////////////
// Abstract Base Class VcComClientWrapperCallback
//
// Provides a means for the VcCom to call your application with messages.
// You must inherit from this class and provide implimentation for the
// functions below.
class VcComClientWrapperCallback
{
    public:

		// SendToAll
		// Called when another application sends a message to all
		// other applications connected to the VcCom.
		// Where
		//	fromName = the name of the application that sent the message
		//	fromId = the Id of the application that sent the message
		//	message = the message
        virtual void SendToAll(
                const std::wstring & fromName,		
                const std::wstring & fromId,
                const std::wstring & message) = 0;

		// SendToName
		// Called when another application sends a message to specified
		// named applications connected to the VcCom.
		// Where
		//	fromName = the name of the application that sent the message
		//	fromId = the Id of the application that sent the message
		//	message = the message
        virtual void SendToName(
                const std::wstring & fromName,
                const std::wstring & fromId,
                const std::wstring & message) = 0;

		// SendToId
		// Called when another application sends a message to specified
		// application connected to the VcCom.
		// Where
		//	fromName = the name of the application that sent the message
		//	fromId = the Id of the application that sent the message
		//	message = the message
        virtual void SendToId(
                const std::wstring & fromName,
                const std::wstring & fromId,
                const std::wstring & message) = 0;
};

/////////////////////////////////////////////////////////////////////
// Class VcComClientWrapper
//
// Provides a means to allow you application to send messages into 
// the VcCom
class VcComClientWrapper :
    private VCCOMLib::IVcComClientCallback
{
    public:

		// Default constructor / destructor
        VcComClientWrapper();
        virtual ~VcComClientWrapper();

        // Connects to the VcCom
		// Where
		//	VcComClientWrapperCallback = the class you have created to receive 
		//								 messages back from the VcCom
        void Connect(VcComClientWrapperCallback*);

		// Disconnects from the VcCom
        void Disconnect();

		// Sends a message to all applications connected to the VcCom
		// Where
		//	message - the message you want to send
        void SendToAll(const std::wstring & message);

		// Sends a message to a named application connected to the VcCom
		// Where
		//	toName - the name of the application you want to send this message to
		//	message - the message you want to send
        void SendToName(
                const std::wstring & toName,
                const std::wstring & message);

		// Sends a message to a specified (Id) application connected to the VcCom
		// Where
		//	toId - the Id of the application you want to send this message to
		//	message - the message you want to send
        void SendToId(
                const std::wstring & toId,
                const std::wstring & message);

		// Set the name of the application to the passed name.
		void Name( const std::wstring &name );

		// Set the name of the application to the passed name.
		std::wstring Name( void );

		// Get the VcCom ID for this application
		std::wstring Id( void );

		////////////////////////////////////////////////////////////////////////
		// The following code allows the VcCom to call this application
        // IVcComClientCallback
        // called from the hub via COM
        STDMETHOD_(ULONG,AddRef)();
        STDMETHOD_(ULONG,Release)();
        STDMETHOD(QueryInterface)(REFIID riid,void __RPC_FAR *__RPC_FAR *ppvObject);
        STDMETHOD(raw_SendToAll)(
                BSTR fromName,
                BSTR fromId,
                BSTR message );
        STDMETHOD(raw_SendToName)(
                BSTR fromName,
                BSTR fromId,
                BSTR toName,
                BSTR message );
        STDMETHOD(raw_SendToId)(
                BSTR fromName,
                BSTR fromId,
                BSTR toId,
                BSTR message );

    private:
		// Holds your object to call when messages arrive
        VcComClientWrapperCallback * m_callback;

		// Pointer to VcCom (VcCom Hook)
        VCCOMLib::IVcComClientPtr m_pvc;
};